#include <stdio.h>
#include <stdint.h>
#include "font_types.h"


#define SPILED_REG_BASE    0xffffc100

#define SPILED_REG_LED_LINE_o           0x004
#define SPILED_REG_LED_RGB1_o           0x010
#define SPILED_REG_LED_RGB2_o           0x014
#define SPILED_REG_LED_KBDWR_DIRECT_o   0x018

#define SPILED_REG_KBDRD_KNOBS_DIRECT_o 0x020
#define SPILED_REG_KNOBS_8BIT_o         0x024


typedef struct fb_t {
  volatile uint16_t *data;
  int width;
  int height;
} fb_t;


fb_t fb_data = {
  .data = (volatile uint16_t *)0xffe00000,
  .width = 480,
  .height = 320,
};

void draw_pixel(fb_t *fb, int x, int y, int c)
{
  if ((unsigned)x >= fb->width)
    return;
  if ((unsigned)y >= fb->height)
    return;
  fb->data[x + fb->width * y] = c;
}

int char_width(font_descriptor_t* fdes, int ch)
{
  if (ch < fdes->firstchar)
    return 0;
  ch -= fdes->firstchar;
  if (ch >= fdes->size)
    return 0;
  if (!fdes->width)
    return fdes->maxwidth;
  return fdes->width[ch];
}

void draw_char(fb_t *fb, font_descriptor_t* fdes, int ch, int x, int y, int c, int back)
{
  int w;
  int bw;
  const uint16_t *b;
  int i;
  int j;
  unsigned patt = 0;

  if (ch < fdes->firstchar)
    return;
  ch -= fdes->firstchar;
  if (ch >= fdes->size)
    return;
  if (!fdes->width)
    w = fdes->maxwidth;
  else
    w = fdes->width[ch];
  bw = (w + 15) / 16;

  if (!fdes->offset)
    b = fdes->bits + ch * bw * fdes->height;
  else
    b = fdes->bits + fdes->offset[ch];

  for (j = 0; j < fdes->height; j++) {
    for (i = 0; i < w; i++) {
      draw_pixel(fb, x + i + 1, y + j + 1, 0x5555);

      if (!(i & 15))
	patt = *(b++);
      if ((patt & 0x8000) || back)
        draw_pixel(fb, x + i, y + j, patt & 0x8000? c: 0);
      patt <<= 1;
    }
  }
}

const char text[] = "I love the word!";

int main(void)
{

  fb_t *fb = &fb_data;
  int x = 10;
  int y = 32;
  const char *p;
  font_descriptor_t* fdes = &font_winFreeSystem14x16;;
  
  for (p = text; *p; p++) {
    draw_char(fb, fdes, *p, x, y, 0xffff, 1);
    x += char_width(fdes, *p);
  }

  return 0;
}
